@@ -49,11 +49,11 @@
                                    "date",
                                    "bdate"))
 {
-    rinla.hgid =  inla.trim("Version_20.02.19")
-    inla.hgid  =  inla.trim("Version_20.02.19")
-    date       =  inla.trim("Wed 19 Feb 2020 09:08:50 AM +03")
-    bdate      =  inla.trim("202002190908")
-    version      =  inla.trim("20.02.19")
+    rinla.hgid =  inla.trim("Version_20.04.18")
+    inla.hgid  =  inla.trim("Version_20.04.18")
+    date       =  inla.trim("Sat Apr 18 20:08:00 +03 2020")
+    bdate      =  inla.trim("202004182008")
+    version      =  inla.trim("20.04.18")
     what = match.arg(what)
 
     if (what %in% c("default", "info")) {
